/* This file is included through vue.c and cannot be built directly. */
#ifdef VUEAPI

/* Read a value directly from host memory */
static int32_t busReadMemory(uint8_t *data, int type) {

    /* Host memory is little-endian */
    #ifndef VUE_BIGENDIAN
        switch (type) {
            case VUE_S8 : return *(int8_t   *)data;
            case VUE_U8 : return *(uint8_t  *)data;
            case VUE_S16: return *(int16_t  *)data;
            case VUE_U16: return *(uint16_t *)data;
            case VUE_S32: return *(int32_t  *)data;
        }

    /* Host memory is big-endian */
    #else
        switch (type) {
            case VUE_S8 : return (int8_t  ) data[0];
            case VUE_U8 : return (uint8_t ) data[0];
            case VUE_S16: return (int16_t ) data[0] << 8 | data[1];
            case VUE_U16: return (uint16_t) data[0] << 8 | data[1];
            case VUE_S32: return (int32_t )
                data[0] << 24 | data[1] << 16 | data[2] << 8 | data[3];
        }
    #endif

    return 0; /* Unreachable, but the compiler doesn't know that */
}

/* Perform a read operation */
static int32_t busReadValue(VUE *vue, uint32_t address, int type, vbool debug){
    address &= ~TYPE_SIZES[type] + 1;
    switch (address >> 24 & 7) {
        case 3:           /* Unmapped */
        case 4: return 0; /* Cartridge expansion */
        case 5: return busReadMemory(&vue->wram[address & 8191], type);
        case 6: return vue->sram == NULL ? 0 :
            busReadMemory(&vue->sram[address & (vue->sram_size - 1)], type);
        case 7: return vue->rom  == NULL ? 0 :
            busReadMemory(&vue->rom [address & (vue->rom_size  - 1)], type);
    }
    return debug * 0; /* Unreachable */
}

#endif /* VUEAPI */
