package vue;

// Java emulation core implementation
class JavaVUE extends VUE {

    // Instance fields
    private byte[] rom;  // Cartridge ROM
    private byte[] sram; // Cartridge SRAM



    ///////////////////////////////////////////////////////////////////////////
    //                               Constants                               //
    ///////////////////////////////////////////////////////////////////////////

    // Memory access type sizes
    private static final int[] TYPE_SIZES = { 1, 1, 2, 2, 4 };



    ///////////////////////////////////////////////////////////////////////////
    //                             Constructors                              //
    ///////////////////////////////////////////////////////////////////////////

    // Default constructor
    JavaVUE() {
    }



    ///////////////////////////////////////////////////////////////////////////
    //                            Public Methods                             //
    ///////////////////////////////////////////////////////////////////////////

    // Release any used resources
    public void dispose() { }; // No action needed

    // Retrieve a copy of the ROM data
    public byte[] getROM() {
        var ret = new byte[rom.length];
        System.arraycopy(rom, 0, ret, 0, ret.length);
        return ret;
    }

    // Determine whether the context is native-backed
    public boolean isNative() {
        return false;
    }

    // Perform a read operation
    public boolean read(int address, byte[] data, int offset, int length) {

        // Error checking
        if (data   == null ||
            offset <  0    ||
            length <  0    ||
            length <  0    || length > 0x01000000 ||
            offset + length > data.length
        ) return false;

        // Perform the operation

        return true;
    }

    // Provide new ROM data
    public boolean setROM(byte[] data, int offset, int length) {

        // Error checking
        if (
            data == null  ||
            offset < 0    ||
            length < 1024 || length > 0x01000000 ||
            (length & length - 1) != 0           ||
            offset + length > data.length
        ) return false;

        // Accept the new ROM data
        rom = new byte[length];
        System.arraycopy(data, offset, rom, 0, length);
        return true;
    }

}
